@@ -51,13 +51,11 @@ bin_breaks <- function(breaks, closed = c("right", "left")) {
 bin_breaks_width <- function(x_range, width = NULL, center = NULL,
                              boundary = NULL, closed = c("right", "left")) {
   if (length(x_range) != 2) {
-    cli::cli_abort("{.arg x_range} must have two elements")
+    cli::cli_abort("{.arg x_range} must have two elements.")
   }
 
-  check_number_decimal(width)
-  if (width <= 0) {
-    cli::cli_abort("{.arg binwidth} must be positive")
-  }
+  # binwidth seems to be the argument name supplied to width. (stat-bin and stat-bindot)
+  check_number_decimal(width, min = 0, allow_infinite = FALSE, arg = "binwidth")
 
   if (!is.null(boundary) && !is.null(center)) {
     cli::cli_abort("Only one of {.arg boundary} and {.arg center} may be specified.")
@@ -105,7 +103,7 @@ bin_breaks_width <- function(x_range, width = NULL, center = NULL,
 bin_breaks_bins <- function(x_range, bins = 30, center = NULL,
                             boundary = NULL, closed = c("right", "left")) {
   if (length(x_range) != 2) {
-    cli::cli_abort("{.arg x_range} must have two elements")
+    cli::cli_abort("{.arg x_range} must have two elements.")
   }
 
   check_number_whole(bins, min = 1)
