@@ -59,6 +59,16 @@ NULL
 #'   variables for which margins are to be created.
 #' @param facets `r lifecycle::badge("deprecated")` Please use `rows`
 #'   and `cols` instead.
+#' @param axes Determines which axes will be drawn. When `"margins"`
+#'   (default), axes will be drawn at the exterior margins. `"all_x"` and
+#'   `"all_y"` will draw the respective axes at the interior panels too, whereas
+#'   `"all"` will draw all axes at all panels.
+#' @param axis.labels Determines whether to draw labels for interior axes when
+#'   the `axes` argument is not `"margins"`. When `"all"` (default), all
+#'   interior axes get labels. When `"margins"`, only the exterior axes get
+#'   labels and the interior axes get none. When `"all_x"` or `"all_y"`, only
+#'   draws the labels at the interior axes in the x- or y-direction
+#'   respectively.
 #' @export
 #' @examples
 #' p <- ggplot(mpg, aes(displ, cty)) + geom_point()
@@ -79,6 +89,12 @@ NULL
 #'   facet_grid(cols = vars(cyl)) +
 #'   geom_point(data = df, colour = "red", size = 2)
 #'
+#' # When scales are constant, duplicated axes can be shown with
+#' # or without labels
+#' ggplot(mpg, aes(cty, hwy)) +
+#'   geom_point() +
+#'   facet_grid(year ~ drv, axes = "all", axis.labels = "all_x")
+#'
 #' # Free scales -------------------------------------------------------
 #' # You can also choose whether the scales should be constant
 #' # across all panels (the default), or whether they should be allowed
@@ -112,6 +128,7 @@ facet_grid <- function(rows = NULL, cols = NULL, scales = "fixed",
                        space = "fixed", shrink = TRUE,
                        labeller = "label_value", as.table = TRUE,
                        switch = NULL, drop = TRUE, margins = FALSE,
+                       axes = "margins", axis.labels = "all",
                        facets = deprecated()) {
   # `facets` is deprecated and renamed to `rows`
   if (lifecycle::is_present(facets)) {
@@ -137,8 +154,22 @@ facet_grid <- function(rows = NULL, cols = NULL, scales = "fixed",
     y = any(space %in% c("free_y", "free"))
   )
 
-  if (!is.null(switch) && !switch %in% c("both", "x", "y")) {
-    cli::cli_abort("{.arg switch} must be either {.val both}, {.val x}, or {.val y}")
+  draw_axes <- arg_match0(axes, c("margins", "all_x", "all_y", "all"))
+  draw_axes <- list(
+    x = any(draw_axes %in% c("all_x", "all")),
+    y = any(draw_axes %in% c("all_y", "all"))
+  )
+
+  # Omitting labels is special-cased internally, so even when no internal axes
+  # are to be drawn, register as labelled.
+  axis_labels <- arg_match0(axis.labels, c("margins", "all_x", "all_y", "all"))
+  axis_labels <- list(
+    x = !draw_axes$x || any(axis_labels %in% c("all_x", "all")),
+    y = !draw_axes$y || any(axis_labels %in% c("all_y", "all"))
+  )
+
+  if (!is.null(switch)) {
+    arg_match0(switch, c("both", "x", "y"))
   }
 
   facets_list <- grid_as_facets_list(rows, cols)
@@ -150,7 +181,8 @@ facet_grid <- function(rows = NULL, cols = NULL, scales = "fixed",
     shrink = shrink,
     params = list(rows = facets_list$rows, cols = facets_list$cols, margins = margins,
       free = free, space_free = space_free, labeller = labeller,
-      as.table = as.table, switch = switch, drop = drop)
+      as.table = as.table, switch = switch, drop = drop,
+      draw_axes = draw_axes, axis_labels = axis_labels)
   )
 }
 
@@ -159,7 +191,7 @@ grid_as_facets_list <- function(rows, cols) {
   is_rows_vars <- is.null(rows) || is_quosures(rows)
   if (!is_rows_vars) {
     if (!is.null(cols)) {
-      msg <- "{.arg rows} must be {.val NULL} or a {.fn vars} list if {.arg cols} is a {.fn vars} list"
+      msg <- "{.arg rows} must be {.code NULL} or a {.fn vars} list if {.arg cols} is a {.fn vars} list."
       # Native pipe have higher precedence than + so any type of gg object can be
       # expected here, not just ggplot
       if (inherits(rows, "gg")) {
@@ -173,7 +205,7 @@ grid_as_facets_list <- function(rows, cols) {
     # For backward-compatibility
     facets_list <- as_facets_list(rows)
     if (length(facets_list) > 2L) {
-      cli::cli_abort("A grid facet specification can't have more than two dimensions")
+      cli::cli_abort("A grid facet specification can't have more than two dimensions.")
     }
     # Fill with empty quosures
     facets <- list(rows = quos(), cols = quos())
@@ -206,7 +238,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     dups <- intersect(names(rows), names(cols))
     if (length(dups) > 0) {
       cli::cli_abort(c(
-              "Faceting variables can only appear in {.arg rows} or {.arg cols}, not both.\n",
+              "Faceting variables can only appear in {.arg rows} or {.arg cols}, not both.",
         "i" = "Duplicated variables: {.val {dups}}"
       ), call = call2(snake_class(self)))
     }
@@ -303,11 +335,29 @@ FacetGrid <- ggproto("FacetGrid", Facet,
   },
   draw_panels = function(panels, layout, x_scales, y_scales, ranges, coord, data, theme, params) {
     if ((params$free$x || params$free$y) && !coord$is_free()) {
-      cli::cli_abort("{.fn {snake_class(coord)}} doesn't support free scales")
+      cli::cli_abort("{.fn {snake_class(coord)}} doesn't support free scales.")
     }
 
-    cols <- which(layout$ROW == 1)
-    rows <- which(layout$COL == 1)
+    # Fill missing parameters for backward compatibility
+    params$draw_axes   <- params$draw_axes   %||% list(x = FALSE, y = FALSE)
+    params$axis_labels <- params$axis_labels %||% list(x = TRUE,  y = TRUE)
+
+    if (!params$axis_labels$x) {
+      cols <- seq_len(nrow(layout))
+      x_axis_order <- as.integer(layout$PANEL[order(layout$ROW, layout$COL)])
+    } else {
+      cols <- which(layout$ROW == 1)
+      x_axis_order <- layout$COL
+    }
+    if (!params$axis_labels$y) {
+      rows <- seq_len(nrow(layout))
+      y_axis_order <- as.integer(layout$PANEL[order(layout$ROW, layout$COL)])
+    } else {
+      rows <- which(layout$COL == 1)
+      y_axis_order <- layout$ROW
+    }
+
+    ranges <- censor_labels(ranges, layout, params$axis_labels)
     axes <- render_axes(ranges[cols], ranges[rows], coord, theme, transpose = TRUE)
 
     col_vars <- unique0(layout[names(params$cols)])
@@ -321,11 +371,9 @@ FacetGrid <- ggproto("FacetGrid", Facet,
 
     aspect_ratio <- theme$aspect.ratio
     if (!is.null(aspect_ratio) && (params$space_free$x || params$space_free$y)) {
-      cli::cli_abort("Free scales cannot be mixed with a fixed aspect ratio")
-    }
-    if (is.null(aspect_ratio) && !params$free$x && !params$free$y) {
-      aspect_ratio <- coord$aspect(ranges[[1]])
+      cli::cli_abort("Free scales cannot be mixed with a fixed aspect ratio.")
     }
+    aspect_ratio <- aspect_ratio %||% coord$aspect(ranges[[1]])
     if (is.null(aspect_ratio)) {
       aspect_ratio <- 1
       respect <- FALSE
@@ -334,7 +382,8 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     }
     ncol <- max(layout$COL)
     nrow <- max(layout$ROW)
-    panel_table <- matrix(panels, nrow = nrow, ncol = ncol, byrow = TRUE)
+    mtx <- function(x) matrix(x, nrow = nrow, ncol = ncol, byrow = TRUE)
+    panel_table <- mtx(panels)
 
     # @kohske
     # Now size of each panel is calculated using PANEL$ranges, which is given by
@@ -358,7 +407,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     }
 
     panel_table <- gtable_matrix("layout", panel_table,
-      panel_widths, panel_heights, respect = respect, clip = coord$clip, z = matrix(1, ncol = ncol, nrow = nrow))
+      panel_widths, panel_heights, respect = respect, clip = coord$clip, z = mtx(1))
     panel_table$layout$name <- paste0('panel-', rep(seq_len(nrow), ncol), '-', rep(seq_len(ncol), each = nrow))
 
     panel_table <- gtable_add_col_space(panel_table,
@@ -367,21 +416,27 @@ FacetGrid <- ggproto("FacetGrid", Facet,
       theme$panel.spacing.y %||% theme$panel.spacing)
 
     # Add axes
-    axis_height_top <- max_height(axes$x$top)
-    axis_height_bottom <- max_height(axes$x$bottom)
-    axis_width_left <- max_width(axes$y$left)
-    axis_width_right <- max_width(axes$y$right)
-    panel_table <- gtable_add_rows(panel_table, axis_height_top, 0)
-    panel_table <- gtable_add_rows(panel_table, axis_height_bottom, -1)
-    panel_table <- gtable_add_cols(panel_table, axis_width_left, 0)
-    panel_table <- gtable_add_cols(panel_table, axis_width_right, -1)
-    panel_pos_col <- panel_cols(panel_table)
-    panel_pos_rows <- panel_rows(panel_table)
+    if (params$draw_axes$x) {
+      axes$x <- lapply(axes$x, function(x) mtx(x[x_axis_order]))
+      panel_table <- weave_axes(panel_table, axes$x)$panels
+    } else {
+      panel_table <- gtable_add_rows(panel_table, max_height(axes$x$top), 0)
+      panel_table <- gtable_add_rows(panel_table, max_height(axes$x$bottom), -1)
+      panel_pos_col <- panel_cols(panel_table)
+      panel_table <- gtable_add_grob(panel_table, axes$x$top, 1, panel_pos_col$l, clip = "off", name = paste0("axis-t-", seq_along(axes$x$top)), z = 3)
+      panel_table <- gtable_add_grob(panel_table, axes$x$bottom, -1, panel_pos_col$l, clip = "off", name = paste0("axis-b-", seq_along(axes$x$bottom)), z = 3)
+    }
 
-    panel_table <- gtable_add_grob(panel_table, axes$x$top, 1, panel_pos_col$l, clip = "off", name = paste0("axis-t-", seq_along(axes$x$top)), z = 3)
-    panel_table <- gtable_add_grob(panel_table, axes$x$bottom, -1, panel_pos_col$l, clip = "off", name = paste0("axis-b-", seq_along(axes$x$bottom)), z = 3)
-    panel_table <- gtable_add_grob(panel_table, axes$y$left, panel_pos_rows$t, 1, clip = "off", name = paste0("axis-l-", seq_along(axes$y$left)), z = 3)
-    panel_table <- gtable_add_grob(panel_table, axes$y$right, panel_pos_rows$t, -1, clip = "off", name = paste0("axis-r-", seq_along(axes$y$right)), z= 3)
+    if (params$draw_axes$y) {
+      axes$y <- lapply(axes$y, function(y) mtx(y[y_axis_order]))
+      panel_table <- weave_axes(panel_table, axes$y)$panels
+    } else {
+      panel_table <- gtable_add_cols(panel_table, max_width(axes$y$left), 0)
+      panel_table <- gtable_add_cols(panel_table, max_width(axes$y$right), -1)
+      panel_pos_rows <- panel_rows(panel_table)
+      panel_table <- gtable_add_grob(panel_table, axes$y$left, panel_pos_rows$t, 1, clip = "off", name = paste0("axis-l-", seq_along(axes$y$left)), z = 3)
+      panel_table <- gtable_add_grob(panel_table, axes$y$right, panel_pos_rows$t, -1, clip = "off", name = paste0("axis-r-", seq_along(axes$y$right)), z= 3)
+    }
 
     # Add strips
     switch_x <- !is.null(params$switch) && params$switch %in% c("both", "x")
@@ -392,22 +447,26 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     panel_pos_col <- panel_cols(panel_table)
     if (switch_x) {
       if (!is.null(strips$x$bottom)) {
-        if (inside_x || as.numeric(axis_height_bottom) == 0) {
+        if (inside_x) {
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$bottom), -2)
           panel_table <- gtable_add_grob(panel_table, strips$x$bottom, -2, panel_pos_col$l, clip = "on", name = paste0("strip-b-", seq_along(strips$x$bottom)), z = 2)
         } else {
-          panel_table <- gtable_add_rows(panel_table, strip_padding, -1)
+          if (!all(vapply(axes$x$bottom, is.zero, logical(1)))) {
+            panel_table <- gtable_add_rows(panel_table, strip_padding, -1)
+          }
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$bottom), -1)
           panel_table <- gtable_add_grob(panel_table, strips$x$bottom, -1, panel_pos_col$l, clip = "on", name = paste0("strip-b-", seq_along(strips$x$bottom)), z = 2)
         }
       }
     } else {
       if (!is.null(strips$x$top)) {
-        if (inside_x || as.numeric(axis_height_top) == 0) {
+        if (inside_x) {
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$top), 1)
           panel_table <- gtable_add_grob(panel_table, strips$x$top, 2, panel_pos_col$l, clip = "on", name = paste0("strip-t-", seq_along(strips$x$top)), z = 2)
         } else {
-          panel_table <- gtable_add_rows(panel_table, strip_padding, 0)
+          if (!all(vapply(axes$x$top, is.zero, logical(1)))) {
+            panel_table <- gtable_add_rows(panel_table, strip_padding, 0)
+          }
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$top), 0)
           panel_table <- gtable_add_grob(panel_table, strips$x$top, 1, panel_pos_col$l, clip = "on", name = paste0("strip-t-", seq_along(strips$x$top)), z = 2)
         }
@@ -416,22 +475,26 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     panel_pos_rows <- panel_rows(panel_table)
     if (switch_y) {
       if (!is.null(strips$y$left)) {
-        if (inside_y || as.numeric(axis_width_left) == 0) {
+        if (inside_y) {
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$left), 1)
           panel_table <- gtable_add_grob(panel_table, strips$y$left, panel_pos_rows$t, 2, clip = "on", name = paste0("strip-l-", seq_along(strips$y$left)), z = 2)
         } else {
-          panel_table <- gtable_add_cols(panel_table, strip_padding, 0)
+          if (!all(vapply(axes$y$left, is.zero, logical(1)))) {
+            panel_table <- gtable_add_cols(panel_table, strip_padding, 0)
+          }
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$left), 0)
           panel_table <- gtable_add_grob(panel_table, strips$y$left, panel_pos_rows$t, 1, clip = "on", name = paste0("strip-l-", seq_along(strips$y$left)), z = 2)
         }
       }
     } else {
       if (!is.null(strips$y$right)) {
-        if (inside_y || as.numeric(axis_width_right) == 0) {
+        if (inside_y) {
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$right), -2)
           panel_table <- gtable_add_grob(panel_table, strips$y$right, panel_pos_rows$t, -2, clip = "on", name = paste0("strip-r-", seq_along(strips$y$right)), z = 2)
         } else {
-          panel_table <- gtable_add_cols(panel_table, strip_padding, -1)
+          if (!all(vapply(axes$y$right, is.zero, logical(1)))) {
+            panel_table <- gtable_add_cols(panel_table, strip_padding, -1)
+          }
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$right), -1)
           panel_table <- gtable_add_grob(panel_table, strips$y$right, panel_pos_rows$t, -1, clip = "on", name = paste0("strip-r-", seq_along(strips$y$right)), z = 2)
         }
