@@ -441,25 +441,37 @@ el_def <- function(class = NULL, inherit = NULL, description = NULL) {
   axis.line.y         = el_def("element_line", "axis.line"),
   axis.line.y.left    = el_def("element_line", "axis.line.y"),
   axis.line.y.right   = el_def("element_line", "axis.line.y"),
+  axis.line.theta     = el_def("element_line", "axis.line.x"),
+  axis.line.r         = el_def("element_line", "axis.line.y"),
+
   axis.text.x         = el_def("element_text", "axis.text"),
   axis.text.x.top     = el_def("element_text", "axis.text.x"),
   axis.text.x.bottom  = el_def("element_text", "axis.text.x"),
   axis.text.y         = el_def("element_text", "axis.text"),
   axis.text.y.left    = el_def("element_text", "axis.text.y"),
   axis.text.y.right   = el_def("element_text", "axis.text.y"),
+  axis.text.theta     = el_def("element_text", "axis.text.x"),
+  axis.text.r         = el_def("element_text", "axis.text.y"),
+
   axis.ticks.length   = el_def("unit"),
-  axis.ticks.length.x = el_def("unit", "axis.ticks.length"),
-  axis.ticks.length.x.top = el_def("unit", "axis.ticks.length.x"),
-  axis.ticks.length.x.bottom = el_def("unit", "axis.ticks.length.x"),
-  axis.ticks.length.y  = el_def("unit", "axis.ticks.length"),
-  axis.ticks.length.y.left = el_def("unit", "axis.ticks.length.y"),
-  axis.ticks.length.y.right = el_def("unit", "axis.ticks.length.y"),
+  axis.ticks.length.x = el_def(c("unit", "rel"), "axis.ticks.length"),
+  axis.ticks.length.x.top = el_def(c("unit", "rel"), "axis.ticks.length.x"),
+  axis.ticks.length.x.bottom = el_def(c("unit", "rel"), "axis.ticks.length.x"),
+  axis.ticks.length.y  = el_def(c("unit", "rel"), "axis.ticks.length"),
+  axis.ticks.length.y.left = el_def(c("unit", "rel"), "axis.ticks.length.y"),
+  axis.ticks.length.y.right = el_def(c("unit", "rel"), "axis.ticks.length.y"),
+  axis.ticks.length.theta = el_def(c("unit", "rel"), "axis.ticks.length.x"),
+  axis.ticks.length.r = el_def(c("unit", "rel"), "axis.ticks.length.y"),
+
   axis.ticks.x        = el_def("element_line", "axis.ticks"),
   axis.ticks.x.top    = el_def("element_line", "axis.ticks.x"),
   axis.ticks.x.bottom = el_def("element_line", "axis.ticks.x"),
   axis.ticks.y        = el_def("element_line", "axis.ticks"),
   axis.ticks.y.left   = el_def("element_line", "axis.ticks.y"),
   axis.ticks.y.right  = el_def("element_line", "axis.ticks.y"),
+  axis.ticks.theta    = el_def("element_line", "axis.ticks.x"),
+  axis.ticks.r        = el_def("element_line", "axis.ticks.y"),
+
   axis.title.x        = el_def("element_text", "axis.title"),
   axis.title.x.top    = el_def("element_text", "axis.title.x"),
   axis.title.x.bottom = el_def("element_text", "axis.title.x"),
@@ -467,21 +479,83 @@ el_def <- function(class = NULL, inherit = NULL, description = NULL) {
   axis.title.y.left   = el_def("element_text", "axis.title.y"),
   axis.title.y.right  = el_def("element_text", "axis.title.y"),
 
+  axis.minor.ticks.x.top    = el_def("element_line", "axis.ticks.x.top"),
+  axis.minor.ticks.x.bottom = el_def("element_line", "axis.ticks.x.bottom"),
+  axis.minor.ticks.y.left   = el_def("element_line", "axis.ticks.y.left"),
+  axis.minor.ticks.y.right  = el_def("element_line", "axis.ticks.y.right"),
+  axis.minor.ticks.theta    = el_def("element_line", "axis.ticks.theta"),
+  axis.minor.ticks.r        = el_def("element_line", "axis.ticks.r"),
+
+  axis.minor.ticks.length = el_def(c("unit", "rel")),
+  axis.minor.ticks.length.x = el_def(c("unit", "rel"), "axis.minor.ticks.length"),
+  axis.minor.ticks.length.x.top = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.x", "axis.ticks.length.x.top")
+  ),
+  axis.minor.ticks.length.x.bottom = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.x", "axis.ticks.length.x.bottom")
+  ),
+  axis.minor.ticks.length.y = el_def(c("unit", "rel"), "axis.minor.ticks.length"),
+  axis.minor.ticks.length.y.left = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.y", "axis.ticks.length.y.left")
+  ),
+  axis.minor.ticks.length.y.right = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.y", "axis.ticks.length.y.right")
+  ),
+  axis.minor.ticks.length.theta = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.x", "axis.ticks.length.theta"),
+  ),
+  axis.minor.ticks.length.r = el_def(
+    c("unit", "rel"), c("axis.minor.ticks.length.y", "axis.ticks.length.r")
+  ),
+
   legend.background   = el_def("element_rect", "rect"),
   legend.margin       = el_def("margin"),
   legend.spacing      = el_def("unit"),
-  legend.spacing.x     = el_def("unit", "legend.spacing"),
-  legend.spacing.y     = el_def("unit", "legend.spacing"),
-  legend.key          = el_def("element_rect", "rect"),
-  legend.key.height   = el_def("unit", "legend.key.size"),
-  legend.key.width    = el_def("unit", "legend.key.size"),
+  legend.spacing.x     = el_def(c("unit", "rel"), "legend.spacing"),
+  legend.spacing.y     = el_def(c("unit", "rel"), "legend.spacing"),
+  legend.key          = el_def("element_rect", "panel.background"),
+  legend.key.height   = el_def(c("unit", "rel"), "legend.key.size"),
+  legend.key.width    = el_def(c("unit", "rel"), "legend.key.size"),
+  legend.key.spacing  = el_def("unit"),
+  legend.key.spacing.x = el_def(c("unit", "rel"), "legend.key.spacing"),
+  legend.key.spacing.y = el_def(c("unit", "rel"), "legend.key.spacing"),
+  legend.frame        = el_def("element_rect", "rect"),
+  legend.axis.line    = el_def("element_line", "line"),
+  legend.ticks        = el_def("element_line", "legend.axis.line"),
+  legend.ticks.length = el_def(c("rel", "unit"), "legend.key.size"),
   legend.text         = el_def("element_text", "text"),
-  legend.text.align   = el_def("numeric"),
+  legend.text.position = el_def("character"),
   legend.title        = el_def("element_text", "title"),
-  legend.title.align  = el_def("numeric"),
-  legend.position     = el_def(c("character", "numeric")),
+  legend.title.position = el_def("character"),
+  legend.byrow        = el_def("logical"),
+  legend.position     = el_def("character"),
+  legend.position.inside = el_def(c("numeric", "integer")),
   legend.direction    = el_def("character"),
-  legend.justification = el_def(c("character", "numeric")),
+
+  legend.justification = el_def(c("character", "numeric", "integer")),
+  legend.justification.top = el_def(
+    c("character", "numeric", "integer"),
+    "legend.justification"
+  ),
+  legend.justification.bottom = el_def(
+    c("character", "numeric", "integer"),
+    "legend.justification"
+  ),
+  legend.justification.left = el_def(
+    c("character", "numeric", "integer"),
+    "legend.justification"
+  ),
+  legend.justification.right = el_def(
+    c("character", "numeric", "integer"),
+    "legend.justification"
+  ),
+  legend.justification.inside = el_def(
+    c("character", "numeric", "integer"),
+    "legend.justification"
+  ),
+
+  legend.location     = el_def("character"),
+
   legend.box          = el_def("character"),
   legend.box.just     = el_def("character"),
   legend.box.margin   = el_def("margin"),
@@ -491,8 +565,8 @@ el_def <- function(class = NULL, inherit = NULL, description = NULL) {
   panel.background    = el_def("element_rect", "rect"),
   panel.border        = el_def("element_rect", "rect"),
   panel.spacing       = el_def("unit"),
-  panel.spacing.x     = el_def("unit", "panel.spacing"),
-  panel.spacing.y     = el_def("unit", "panel.spacing"),
+  panel.spacing.x     = el_def(c("unit", "rel"), "panel.spacing"),
+  panel.spacing.y     = el_def(c("unit", "rel"), "panel.spacing"),
   panel.grid.major.x  = el_def("element_line", "panel.grid.major"),
   panel.grid.major.y  = el_def("element_line", "panel.grid.major"),
   panel.grid.minor.x  = el_def("element_line", "panel.grid.minor"),
@@ -522,10 +596,11 @@ el_def <- function(class = NULL, inherit = NULL, description = NULL) {
   plot.caption        = el_def("element_text", "title"),
   plot.caption.position = el_def("character"),
   plot.tag            = el_def("element_text", "title"),
-  plot.tag.position   = el_def(c("character", "numeric")),  # Need to also accept numbers
+  plot.tag.position   = el_def(c("character", "numeric", "integer")),  # Need to also accept numbers
+  plot.tag.location   = el_def("character"),
   plot.margin         = el_def("margin"),
 
-  aspect.ratio        = el_def("numeric")
+  aspect.ratio        = el_def(c("numeric", "integer"))
 )
 
 # Check that an element object has the proper class
