@@ -4,15 +4,17 @@ test_that("data is ordered by x", {
   ps <- ggplot(df, aes(x, y))+
     geom_smooth(stat = "identity", se = FALSE)
 
-  expect_equal(layer_data(ps)[c("x", "y")], df[order(df$x), ])
+  expect_equal(layer_data(ps)[c("x", "y")], df[order(df$x), ], ignore_attr = TRUE)
 })
 
 test_that("geom_smooth works in both directions", {
-  p <- ggplot(mpg, aes(displ, hwy)) + geom_smooth()
+  p <- ggplot(mpg, aes(displ, hwy)) +
+    geom_smooth(method = 'loess', formula = y ~ x)
   x <- layer_data(p)
   expect_false(x$flipped_aes[1])
 
-  p <- ggplot(mpg, aes(hwy, displ)) + geom_smooth(orientation = "y")
+  p <- ggplot(mpg, aes(hwy, displ)) +
+    geom_smooth(orientation = "y", method = 'loess', formula = y ~ x)
   y <- layer_data(p)
   expect_true(y$flipped_aes[1])
 
@@ -77,6 +79,22 @@ test_that("default smoothing methods for small and large data sets work", {
   expect_equal(plot_data$y, as.numeric(out))
 })
 
+test_that("geom_smooth() works when one group fails", {
+  # Group A fails, B succeeds
+  df <- data_frame0(
+    x = c(1, 2, 1, 2, 3),
+    y = c(1, 2, 3, 2, 1),
+    g = rep(c("A", "B"), 2:3)
+  )
+  p <- ggplot(df, aes(x, y, group = g)) +
+    geom_smooth(method = "loess", formula = y ~ x)
+
+  suppressWarnings(
+    expect_warning(ld <- layer_data(p), "Failed to fit group 1")
+  )
+  expect_equal(unique(ld$group), 2)
+  expect_gte(nrow(ld), 2)
+})
 
 # Visual tests ------------------------------------------------------------
 
@@ -87,11 +105,11 @@ test_that("geom_smooth() works with alternative stats", {
 
   expect_doppelganger("ribbon turned on in geom_smooth", {
     ggplot(df, aes(x, y, color = fill, fill = fill)) +
-      geom_smooth(stat = "summary") # ribbon on by default
+      geom_smooth(stat = "summary", fun.data = mean_se) # ribbon on by default
   })
 
   expect_doppelganger("ribbon turned off in geom_smooth", {
     ggplot(df, aes(x, y, color = fill, fill = fill)) +
-      geom_smooth(stat = "summary", se = FALSE) # ribbon is turned off via `se = FALSE`
+      geom_smooth(stat = "summary", se = FALSE, fun.data = mean_se) # ribbon is turned off via `se = FALSE`
   })
 })
